<?php

namespace App\Controller\OgRepository\Webhook;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Entity\Image;
use App\Entity\ImageRepository;
use App\Entity\Trace;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

#[AsController]
class ResponseController extends AbstractOgRepositoryController
{
    /**
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    #[Route('/og-repository/webhook', name: 'og_repository_webhook', methods: ['POST'])]
    public function repositoryWebhook(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);

        if (!isset($data['job_id'])) {
            return new JsonResponse(['message' => 'Invalid request'], Response::HTTP_BAD_REQUEST);
        }

        $action = $data['job_id'];

        if (str_starts_with($action, "CreateAuxiliarFiles_")) {
            $this->handleCreateAuxFiles($data);
        } elseif (str_starts_with($action, "ExportImage_")) {
            $this->processImageAction($data, 'export');
        } elseif (str_starts_with($action, "ImportImage_")) {
            $this->processImageAction($data, 'import');
        } else {
            return new JsonResponse(['message' => 'Invalid action'], Response::HTTP_BAD_REQUEST);
        }

        return new JsonResponse($data, Response::HTTP_OK);
    }

    private function handleCreateAuxFiles(array $data): void
    {
        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);
        $imageUuid = $trace->getInput()['imageUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);

        if ($image === null) {
            $this->updateTraceStatus($trace, TraceStatus::FAILED, 'Image not found');
            return;
        }

        $image->setImageFullsum($data['image_id']);
        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);

        $this->updateTraceStatus($trace, TraceStatus::SUCCESS);
    }


    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    private function processImageAction(array $data, string $actionType): void
    {
        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);
        $imageUuid = $trace->getInput()['imageUuid'];
        $repositoryUuid = $trace->getInput()['repositoryUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);
        $repository = $this->entityManager->getRepository(ImageRepository::class)->findOneBy(['uuid' => $repositoryUuid]);

        if ($image === null) {
            $this->updateTraceStatus($trace, TraceStatus::FAILED, 'Image not found');
            return;
        }

        $this->logger->info("Image $actionType", ['image' => $image->getName()]);

        $params = [
            'json' => [
                'image' => $image->getName().'.img'
            ]
        ];

        $this->logger->info('Creating aux files', ['image' => $image->getName()]);
        $content = $this->createRequest('POST', 'http://'.$repository->getIp().':8006/ogrepository/v1/images/torrentsum', $params);

        $inputData = [
            'imageName' => $image->getName(),
            'imageUuid' => $image->getUuid(),
        ];

        $this->createService->__invoke($image->getClient(), CommandTypes::CREATE_IMAGE_AUX_FILE, TraceStatus::IN_PROGRESS, $content['job_id'], $inputData);

        $image->setRepository($repository);
        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);

        $this->updateTraceStatus($trace, TraceStatus::SUCCESS);
    }

    private function updateTraceStatus(Trace $trace, string $status, string $output = null): void
    {
        $trace->setStatus($status);
        $trace->setFinishedAt(new \DateTime());

        if ($output !== null) {
            $trace->setOutput($output);
        }

        $this->entityManager->persist($trace);
        $this->entityManager->flush();

        if ($status === TraceStatus::FAILED) {
            new JsonResponse(['message' => $output], Response::HTTP_NOT_FOUND);
        }
    }
}
