<?php

namespace App\Dto\Input;

use ApiPlatform\Metadata\ApiProperty;
use App\Dto\Output\ClientOutput;
use App\Dto\Output\ImageRepositoryOutput;
use App\Dto\Output\OrganizationalUnitOutput;
use App\Dto\Output\PartitionOutput;
use App\Dto\Output\RemoteCalendarRuleOutput;
use App\Dto\Output\SoftwareProfileOutput;
use App\Entity\Image;
use App\Entity\OrganizationalUnit;
use App\Entity\Partition;
use App\Model\ImageStatus;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

final class ImageInput
{
    #[Assert\NotBlank(message: 'validators.image.name.not_blank')]
    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The name of the image', example: "Image 1")]
    public ?string $name = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The description of the image', example: "Image 1 description")]
    public ?string $description = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The comments of the image', example: "Image 1 comments")]
    public ?string $comments = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The type of the image', example: "Server")]
    public ?string $type = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The type of the image', example: "Server")]
    public ?string $source = 'input';

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The status of the image', example: "PENDING")]
    public ?string $status = ImageStatus::PENDING;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The software profile of the image')]
    public ?SoftwareProfileOutput $softwareProfile = null;

    /**
     * @var ImageRepositoryOutput[]
     */
    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The image repository of the image')]
    public ?array $imageRepositories = [];

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The client of the image')]
    public ?ClientOutput $client = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The client of the image')]
    public ?PartitionOutput $partition = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The parent of the image')]
    public ?self $parent = null;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The remote pc of the image')]
    public ?bool $remotePc = false;

    #[Groups(['image:write'])]
    #[ApiProperty(description: 'The global property of the image')]
    public ?bool $isGlobal = false;


    public function __construct(?Image $image = null)
    {
        if (!$image) {
            return;
        }

        $this->name = $image->getName();
        $this->description = $image->getDescription();
        $this->comments = $image->getComments();
        $this->type = $image->getType();
        $this->remotePc = $image->isRemotePc();
        $this->isGlobal = $image->isGlobal();
        $this->status = $image->getStatus();

        if ($image->getSoftwareProfile()) {
            $this->softwareProfile = new SoftwareProfileOutput($image->getSoftwareProfile());
        }

        if ($image->getRepositories()) {
            foreach ($image->getRepositories() as $repository) {
                $this->imageRepositories[] = new ImageRepositoryOutput($repository);
            }
        }

        if ($image->getClient()) {
            $this->client = new ClientOutput($image->getClient());
        }

        if ($image->getParent()) {
            $this->parent = new self($image->getParent());
        }
    }

    public function createOrUpdateEntity(?Image $image = null): Image
    {
        if (!$image) {
            $image = new Image();
            $image->setStatus(ImageStatus::PENDING);
        }

        $image->setName($this->name);
        $image->setDescription($this->description);
        $image->setComments($this->comments);
        $image->setType($this->type);

        if ($this->softwareProfile) {
            $image->setSoftwareProfile($this->softwareProfile->getEntity());
        }

        if ($this->client) {
            $image->setClient($this->client->getEntity());
        }

        if ($this->imageRepositories) {
            foreach ($this->imageRepositories as $repository) {
                $repositoriesToAdd[] = $repository->getEntity();
            }
        }

        $repositoriesToAdd[] = $image->getClient()?->getRepository();
        $image->setRepositories( $repositoriesToAdd ?? [] );

        $image->setRemotePc($this->remotePc);
        $image->setIsGlobal($this->isGlobal);
        $image->setCreated(false);

        $partitionInfo = [];

        if ($this->partition) {
            /** @var Partition $partition */
            $partition = $this->partition->getEntity();
            $partitionInfo["numDisk"] = $partition->getDiskNumber();
            $partitionInfo["numPartition"] = $partition->getPartitionNumber();
            $partitionInfo["partitionCode"] = $partition->getPartitionCode();
            $partitionInfo["filesystem"] = $partition->getFilesystem();
            $partitionInfo["osName"] = $partition->getOperativeSystem()?->getName();
            $image->setPartitionInfo(json_encode($partitionInfo));
        }

        return $image;
    }
}